local draw = require'debugWorldDraw_11'
love.setDeprecationOutput(false)
local la,le,lfs,lf,lg,li,lj,lk,lm,lmo,lp,ls,lsys,lth,lt,lw = require'ns'()

local world
local world_sleep = false
local water_density = 500
local player_density = 1
local box_density = 1
local waterlevel_height = 320
local box_width = 44
local box_height = 44
local player_height = 38
local player_grounded = false
local player_on_box = false
local player_maxvel = 80
local player_force = 200
local player_brake = 10
local water_viscosity = 2

local function newBody()
  return {body = false, shape = false, fixture = false}
end

local function createRect(entity, x, y, w, h, mode)
  entity.shape = lp.newRectangleShape(w, h)
  entity.body = lp.newBody(world, x + w/2, y + h/2, mode)
  entity.fixture = lp.newFixture(entity.body, entity.shape)
end

local box = newBody()
local player = newBody()
local water = newBody()
local ground1 = newBody()
local ground2 = newBody()
local bottom = newBody()

local box_in_water = false

local function within(a, b, range)
  return math.abs(a - b) <= range
end

local collisions = {}
local revcoll = {}
local ncoll = 0

local function checkPlayerCollisions()
  player_grounded = false
  for i = 1, ncoll do
    local contact = collisions[i]
    local x1, y1, x2, y2 = contact:getPositions()
    if x1 then
      local px, py = player.body:getPosition()
      local f1, f2 = contact:getFixtures()
      local nx, ny = contact:getNormal()
      if f1 == player.fixture then ny = -ny end
      if within(nx, 0, 0.1) and ny < 0 then
        player_grounded = true
      end
    end
  end
end


local function collisionStart(f1, f2, contact)
  if f1 == water.fixture then
    f1, f2 = f2, f1
  end
  if f1 == box.fixture and f2 == water.fixture then
    box_in_water = true
  end
  if f1 == player.fixture or f2 == player.fixture then
    ncoll = ncoll + 1
    collisions[ncoll] = contact
    revcoll[contact] = ncoll
    checkPlayerCollisions()
    if player_grounded and (f1 == box.fixture or f2 == box.fixture) then
      player_on_box = true
    end
  end
end

local function collisionEnd(f1, f2, contact)
  if f1 == water.fixture then
    f1, f2 = f2, f1
  end
  if f1 == box.fixture and f2 == water.fixture then
    box_in_water = false
  end
  if f1 == player.fixture or f2 == player.fixture then
    local index = revcoll[contact]

    revcoll[contact] = nil
    collisions[index] = collisions[ncoll]
    collisions[ncoll] = nil
    if ncoll ~= index then
      revcoll[collisions[index]] = index
    end
    ncoll = ncoll - 1
    checkPlayerCollisions()
    if not player_grounded then
      player_on_box = false
    end
  end
end

function love.load()
  lp.setMeter(20)
  world = lp.newWorld(0, 9.8*20, world_sleep)
  world:setCallbacks(collisionStart, collisionEnd)
  createRect(player, 100, 200, 20, player_height, 'dynamic')
  player.fixture:setDensity(player_density)
  createRect(box, 300, 280, box_width, box_height, 'dynamic')
  box.fixture:setDensity(box_density)
  createRect(water, 240, waterlevel_height, 360, 220)
  water.fixture:setSensor(true)
  createRect(ground1, 0, 300, 240, 240)
  createRect(ground2, 600, 300, 180, 240)
  createRect(bottom, 0, 540, 780, 10)

  box.body:setFixedRotation(true)
  player.body:setFixedRotation(true)
  lg.setBackgroundColor(1, 1, 1)
end

function love.update(dt)
  if (W or 0) < 0.02 then
  world:update(dt)end
  if box_in_water then
    -- buoyancy
    local _tmp, displaced_mass
    _tmp, _tmp, displaced_mass = box.shape:computeMass(water_density)
    local box_y = box.body:getY()
    local box_proportion_in_water = math.max(0,
        math.min(box_y - waterlevel_height + box_height/2, box_height)
      ) / box_height
    box.body:applyForce(0, -box_proportion_in_water * displaced_mass)
    -- damp due to water viscosity
    local vx, vy = box.body:getLinearVelocity()
    box.body:applyForce(-vx*water_viscosity, -vy*water_viscosity)
  end
-- Lock down
--  box.body:setX(300)

  if player_grounded then
    -- can't move if not grounded
    local lkey = lk.isScancodeDown("a")
    local rkey = lk.isScancodeDown("d")
    if lkey ~= rkey then
      local sgn = rkey and 1 or -1
      -- it gets tricky!
      local vx, vy = player.body:getLinearVelocity()
      if vx * sgn < 0 then
        -- Change direction immediately
        vx = 0
        player.body:setLinearVelocity(vx, vy)
      end
      if vx * sgn < player_maxvel then
        player.body:applyForce(player_force * sgn, 0)
      else
        vx = player_maxvel * sgn
        player.body:setLinearVelocity(vx, vy)
      end
      -- is the player on the box?
      if player_on_box then
        -- apply reaction force to the box
        box.body:applyForce(-player_force*sgn/2, 0)
      end
    else
      -- Stop player
      local vx = player.body:getLinearVelocity()
      player.body:applyForce(-vx * player_brake, 0)
    end
  end

end

function love.draw()
  draw(world, 0, 0, 800, 600)
--  lg.setColor(0,0,0)
--  lg.print(tostring(player_grounded) .. " " .. tostring(player_on_box))
end

function love.keypressed(_tmp, k)
  if k == "escape" then return le.quit() end
  if k == "space" and player_grounded then
    player.body:applyLinearImpulse(0, -300)
  end
end
